import tensorflow as tf
import numpy as np
import pandas as pd
from keras_preprocessing.image import load_img
from keras_preprocessing.image import img_to_array
from teachable_machine import TeachableMachine
from sklearn.linear_model import LinearRegression
from response import Response

# Model Saved Path
INSECT_MODEL_PATH = 'models/insect_type_model.h5'
INSECT_MODEL_LABELS_PATH = 'models/insect_type_model_labels.txt'

TYPE_MODEL_PATH = 'models/paddy_or_other_pest_model.h5'
TYPE_MODEL_LABELS_PATH = 'models/paddy_or_other_pest_model_labels.txt'

LEAF_MODEL_PATH = 'models/healthy_disease_leaf_model.h5'
LEAF_MODEL_LABELS_PATH = 'models/healthy_disease_leaf_model_labels.txt'

COLOR_MODEL_PATH = 'models/colors_level_model.h5'
COLOR_MODEL_LABELS_PATH = 'models/colors_level_model_labels.txt'

# Define the mapping from class indices to class labels
type_class_labels = {0: 'PaddyPest', 1: 'OtherPest'}
class_labels = {0: 'rice leaf roller', 1: 'rice leaf caterpillar ', 2: 'paddy stem maggot', 3: 'asiatic rice borer',
                4: 'yellow rice borer', 5: 'rice gall midge ', 6: 'Rice Stemfly', 7: 'brown plant hopper',
                8: 'white backed plant hopper',
                9: 'rice leafhopper', 10: 'Paddy Bug'}
color_labels = {0: 'ColorL2', 1: 'ColorL3', 2: 'ColorL4', 3: 'ColorL5', }
leaf_labels = {0: 'Healthy', 1: 'BrownSpot', 2: 'Hispa', 3: 'LeafBlast', 4: 'Bacterialleafblight'}


def getInsectPrediction(filename):
    typeModel = TeachableMachine(model_path=TYPE_MODEL_PATH,
                                 labels_file_path=TYPE_MODEL_LABELS_PATH)

    image_path = 'uploads/' + filename

    result = typeModel.classify_image(image_path)

    print("class_index", result["class_index"])

    print("class_name:::", result["class_name"])

    print("class_confidence:", result["class_confidence"])

    typeLabel = str(result["class_name"]).rstrip()

    if typeLabel == 'PaddyPest':
        insectModel = TeachableMachine(model_path=INSECT_MODEL_PATH,
                                       labels_file_path=INSECT_MODEL_LABELS_PATH)

        image_path = 'uploads/' + filename

        result = insectModel.classify_image(image_path)

        print("class_index", result["class_index"])

        print("class_name:::", result["class_name"])

        print("class_confidence:", result["class_confidence"])

        insect_label = str(result["class_name"]).rstrip()
        probability = str(result["class_confidence"])

        return insect_label, probability
    else:
        print('Not a paddy pest')
        return typeLabel, 0.0


def getColorPrediction(filename):
    colorModel = TeachableMachine(model_path=COLOR_MODEL_PATH,
                                  labels_file_path=COLOR_MODEL_LABELS_PATH)

    image_path = 'uploads/' + filename

    result = colorModel.classify_image(image_path)

    print("class_index", result["class_index"])

    print("class_name:::", result["class_name"])

    print("class_confidence:", result["class_confidence"])

    colorLabel = str(result["class_name"])
    probability = str(result["class_confidence"])

    return colorLabel, probability


def getLeafPrediction(filename):
    model = TeachableMachine(model_path=LEAF_MODEL_PATH,
                             labels_file_path=LEAF_MODEL_LABELS_PATH)

    image_path = 'uploads/' + filename

    result = model.classify_image(image_path)

    print("class_index", result["class_index"])

    print("class_name:::", result["class_name"])

    print("class_confidence:", result["class_confidence"])

    label = str(result["class_name"])
    probability = str(result["class_confidence"])

    return label, probability


def predictFertilizer(msg_received):
    try:
        type = msg_received["type"]
        year = msg_received["year"]
        month = msg_received["month"]
        keyWord = msg_received["keyWord"]
    except:
        p1 = Response(1, "Required data missing.", None)
        return p1

    type = msg_received["type"]
    year = msg_received["year"]
    month = msg_received["month"]
    keyWord = msg_received["keyWord"]

    data = ()
    if type == 'month':
        data = predictFertilizerMonthly(int(year), int(month), keyWord)
    elif type == 'year':
        data = predictFertilizerYearly(int(year), keyWord).to_dict()
    else:
        data = predictFertilizer(int(year), int(month), keyWord)

    p1 = Response(0, "Success.", data)
    return p1


def predictFertilizerMonthly(year, predMonth, predKeyWord):
    year = year - 1

    # Read the rainfall data into a DataFrame
    df = pd.read_csv('csvFiles/fertilizer1.csv')

    # Convert the date column to datetime and set it as the index
    df['date'] = pd.to_datetime(df['date'])
    df.set_index('date', inplace=True)

    # Create a new DataFrame that contains only the crop data
    crop_df = df[[predKeyWord]]

    # Add a column for the month of each data point
    crop_df['month'] = crop_df.index.month

    # Add a column for the year of each data point
    crop_df['year'] = crop_df.index.year

    # Create a pivot table that shows the total rainfall for each month and year
    rainfall_pivot = crop_df.pivot_table(values=predKeyWord, index='year', columns='month', aggfunc='sum')

    # Create a new DataFrame that contains only the data for the previous year
    prev_year_df = rainfall_pivot.loc[year]

    # Drop the December rainfall data (since we're trying to predict it)
    prev_year_df.drop(predMonth, inplace=True)

    # Flatten the DataFrame to create two arrays (X and y) for the linear regression model
    X = prev_year_df.index.values.reshape(-1, 1)
    y = prev_year_df.values.reshape(-1, 1)

    # Create a linear regression model and fit it to the data
    model = LinearRegression()
    model.fit(X, y)

    # Predict the rainfall for December of the current year
    current_year = pd.Timestamp.now().year
    X_pred = [[current_year]]
    y_pred = model.predict(X_pred)

    # Print the predicted rainfall for December of the current year
    print('Our guess for this year\'s December ' + predKeyWord + ' is:', y_pred[0][0])

    return y_pred[0][0]


def predictFertilizerYearly(year, predKeyWord):
    # Read the rainfall data into a DataFrame
    df = pd.read_csv('csvFiles/fertilizer1.csv')

    # Convert the date column to datetime and set it as the index
    df['date'] = pd.to_datetime(df['date'])
    df.set_index('date', inplace=True)

    # Create a new DataFrame that contains only the crop data
    crop_df = df[[predKeyWord]]

    # Add a column for the month of each data point
    crop_df['month'] = crop_df.index.month

    # Add a column for the year of each data point
    crop_df['year'] = crop_df.index.year

    # Create a pivot table that shows the total rainfall for each month and year
    rainfall_pivot = crop_df.pivot_table(values=predKeyWord, index='year', columns='month', aggfunc='sum')

    # Create a new DataFrame that contains only the data for the previous year
    prev_year_df = rainfall_pivot.loc[year - 1]

    # Drop the December rainfall data (since we're using it as the target variable)
    # prev_year_df.drop(predMonth, inplace=True)

    # Create a DataFrame to store the predictions
    predictions_df = pd.DataFrame(columns=['month', predKeyWord])

    # Loop over each month and make a prediction for the next year's rainfall
    for month in range(1, 13):
        # Create a new DataFrame that contains only the data for the current month
        month_df = prev_year_df[[month]]

        print(month_df)

        # Flatten the DataFrame to create two arrays (X and y) for the linear regression model
        X = month_df.index.values.reshape(-1, 1)
        y = month_df.values.reshape(-1, 1)

        # Create a linear regression model and fit it to the data
        model = LinearRegression()
        model.fit(X, y)

        # Predict the rainfall for the next year
        next_year = year
        X_pred = [[next_year]]
        y_pred = model.predict(X_pred)

        # Add the prediction to the predictions DataFrame
        predictions_df = predictions_df.append({'month': month, predKeyWord: y_pred[0][0]}, ignore_index=True)

    # Print the predicted rainfall for each month of the next year
    return predictions_df
